#include <filter.h>
#include <fract2float_conv.h>
#include <stdlib.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "notes.h"
#include "types.h"
#include "g2mparameters.h"
#include "debug.h"


// static buffers and twiddle table for FFT
static fract16 sample_buffer[FFT_SIZE];
static complex_fract16 temp[FFT_SIZE];
static complex_fract16 fft_result[FFT_SIZE];
static complex_fract16 twiddle[FFT_SIZE];

FILE* input_file = NULL;
static unsigned int fft_count = 0;


bool acquire_and_transform(fract16* in, complex_fract16* out);


int main(int argc, char** argv){
	previous_pitch = SILENCE;

	input_file = fopen(argv[1], "r");

	// twiddle table
	twidfftrad4_fr16(twiddle, FFT_SIZE);

	// cyclic executive
    bool more_to_read = TRUE;
    
	while(more_to_read) {
	    acquire_and_transform(sample_buffer, fft_result);
        pitch_t current_pitch = pitch_from_frequency(get_fundamental(fft_result));
        
		if (is_event(current_pitch)) {
		    int i; for (i = 0; i < DISCARDED_FFTS; i++)
                more_to_read = acquire_and_transform(sample_buffer, fft_result);
            
            // if there's nothing more to read, don't emit anymore notes
            current_pitch = pitch_from_frequency(get_fundamental(fft_result));
            previous_pitch = make_note(fft_count, current_pitch, previous_pitch);
		}
	}

	return 0;
}


bool acquire_and_transform(fract16* in, complex_fract16* out) {
    long position = ftell(input_file);
    
    struct stat fprops;
    fstat(fileno(input_file), &fprops);
    
    if(position + FFT_SIZE > fprops.st_size) {
        exit(-1);
    }
    
    fread(in, SAMPLE_SIZE, FFT_SIZE, input_file);
    
    rfftrad4_fr16(in,temp,out,twiddle,1,FFT_SIZE,0,SCALE_FACTOR);
    fft_count++;
    return TRUE;
}

