#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <arpa/inet.h>
#include "midi.h"
#include "types.h"
#include "notes.h"
#include "midiparameters.h"


static char instrument_timbre[3] = { 0x00, /* channel0 */ 0xc0, MIDI_INSTRUMENT };
static char end_of_track[4] = {0x00, 0xFF, 0x2F, 0x00};

FILE* midi_file = NULL;
int chunk_size = 0;


int main(int argc, char** argv) {
    note_t current_note;
    int last_time = 0;
    
    midi_file = fopen(argv[1],"w");
    
    write_header();
    write_chunk();
    
    while(TRUE) {
        char* current = (char*) &current_note;
        int bytes_read = 1;
        int i; for(i = 0; i < sizeof(note_t) && bytes_read >= 1; i++)
            bytes_read = fread(current++, 1, 1, stdin);

        if(bytes_read < 1)
            break;
            
        write_note_event(
            current_note.pitch,
            (current_note.time - last_time) * TICKS_PER_FFT,
            current_note.on);
        last_time = current_note.time;
    }
    
    write_endoftrack();
    overwrite_chunk();
    
    return 0;
}


void write_header(){
	header_t header;
	memcpy(header.chunk_id, "MThd", 4);
	
	header.chunk_size = htonl(6);
	header.format_type = htons(0);
	header.number_of_tracks = htons(1);
	header.time_division = htons(/* uses TPBeat */ 0 | TICKS_PER_BEAT);
	
	fwrite(&header, HEADER_SIZE, 1, midi_file);
}


void write_chunk() {
	track_chunk_t chunk;
	memcpy(chunk.chunk_id, "MTrk", 4);
	chunk.chunk_size = htonl(0);
	
	fwrite(&chunk, TRACK_CHUNK_SIZE, 1, midi_file);
	
	fwrite(&instrument_timbre, sizeof(instrument_timbre), 1, midi_file);
	chunk_size += sizeof(instrument_timbre);
}


void overwrite_chunk() {
    int bigendian_chunk_size = htonl(chunk_size);
    
    fseek(midi_file, HEADER_SIZE + (4*sizeof(char)), SEEK_SET);
    fwrite(&bigendian_chunk_size, sizeof(int), 1, midi_file);
}


void write_note_event(char note_number, int delta_time, bool on) {
	midi_event_t note;
	note.type_and_channel = (on ? NOTE_ON : NOTE_OFF) << 4;
	note.parameter1 = note_number;
	note.parameter2 = NOTE_VELOCITY;

	write_delta_time(delta_time, midi_file);
	
	fwrite(&note, MIDI_EVENT_SIZE, 1, midi_file);
	chunk_size += MIDI_EVENT_SIZE;
}


void write_delta_time(int delta_time, FILE* output) {
	char current_byte = 0;
	int buffer = delta_time & 0x7F;

	while ((delta_time >>= 7) != 0) {
		buffer <<= 8;
		buffer |= ((delta_time & 0x7F) | 0x80);
	}

	while (TRUE) {
	    current_byte = (char) (buffer & 0xff);
	    fwrite(&current_byte, sizeof(char), 1, output);
	    chunk_size++;
		if ((buffer & 0x80) != 0)
			buffer >>= 8;
		else
			break;
	}
}


void write_endoftrack() {
    fwrite(&end_of_track, sizeof(end_of_track), 1, midi_file);
    chunk_size += sizeof(end_of_track);
}

